#CHE210D Final Project
#Transient gas enrichment at hydrophobic interface

#Python file structure:
    #I.    Module import
    #II.   Function definitions
    #III.  USER INPUT
    #IV.   User input processing
    #V.    Equilibration MD loop
    #VI.   Production MD loop
    
#I. MODULE IMPORT:
#-------------------------------------------------#
#import python modules
import numpy as np
import time
import atomwrite

#import compiled fortran library
import mdlib

import sys
import os


#II. FUNCTION DEFINITIONS:
#-------------------------------------------------#    
def LineSearch(Pos, Dir, dx, EFracTol,
               Accel = 1.5, MaxInc = 10., MaxIter = 10000):
    """Performs a line search along direction Dir.
Input:
    Pos: starting positions, (N,3) array
    Dir: (N,3) array of gradient direction
    dx: initial step amount
    EFracTol: fractional energy tolerance
    Accel: acceleration factor
    MaxInc: the maximum increase in energy for bracketing
    MaxIter: maximum number of iteration steps
Output:
    PEnergy: value of potential energy at minimum along Dir
    Pos: minimum energy (N,3) position array along Dir
"""
    #identify global variables
    global itype, rfix, sigma2ls, epsls, Ntypes, Lx, Ly, Lz, Cut
    
    #start the iteration counter
    Iter = 0

    #find the normalized direction    
    NormDir = np.clip(Dir, -1.e100, 1.e100)
    NormDir = NormDir / np.sqrt(np.sum(NormDir * NormDir))

    #take the first two steps and compute energies    
    Dists = [0., dx]
    PEs = [mdlib.calcenergy(Pos + NormDir * x, rfix, itype, sigma2ls,epsls, \
        Lx, Ly, Lz, Cut) for x in Dists]
    
    #if the second point is not downhill in energy, back
    #off and take a shorter step until we find one
    while PEs[1] > PEs[0]:
        Iter += 1
        dx = dx * 0.5
        Dists[1] = dx
        PEs[1] = mdlib.calcenergy(Pos + NormDir * dx, rfix, itype, sigma2ls, epsls, \
            Lx, Ly, Lz, Cut)
        
    #find a third point
    Dists = Dists + [2. * dx]
    PEs = PEs + [mdlib.calcenergy(Pos + NormDir * 2. * dx, rfix, itype, sigma2ls, epsls, \
        Lx, Ly, Lz, Cut)]
    
    #keep stepping forward until the third point is higher
    #in energy; then we have bracketed a minimum
    while PEs[2] < PEs[1]:
        Iter += 1
            
        #find a fourth point and evaluate energy
        Dists = Dists + [Dists[-1] + dx]
        PEs = PEs + [mdlib.calcenergy(Pos + NormDir * Dists[-1], rfix, itype,sigma2ls,epsls, \
            Lx, Ly, Lz, Cut)]

        #check if we increased too much in energy; if so, back off
        if (PEs[3] - PEs[0]) > MaxInc * (PEs[0] - PEs[2]):
            PEs = PEs[:3]
            Dists = Dists[:3]
            dx = dx * 0.5
        else:
            #shift all of the points over
            PEs = PEs[-3:]
            Dists = Dists[-3:]
            dx = dx * Accel
            
    #we've bracketed a minimum; now we want to find it to high
    #accuracy
    OldPE3 = 1.e300
    while True:
        Iter += 1
        if Iter > MaxIter:
            print "Warning: maximum number of iterations reached in line search."
            break
            
        #store distances for ease of code-reading
        d0, d1, d2 = Dists
        PE0, PE1, PE2 = PEs

        #use a parobolic approximation to estimate the location
        #of the minimum
        d10 = d0 - d1
        d12 = d2 - d1
        Num = d12*d12*(PE0-PE1) - d10*d10*(PE2-PE1)
        Dem = d12*(PE0-PE1) - d10*(PE2-PE1)
        if Dem == 0:
            #parabolic extrapolation won't work; set new dist = 0 
            d3 = 0
        else:
            #location of parabolic minimum
            d3 = d1 + 0.5 * Num / Dem
            
        #compute the new potential energy
        PE3 = mdlib.calcenergy(Pos + NormDir * d3, rfix, itype, sigma2ls, epsls, \
            Lx, Ly, Lz, Cut)
        
        #sometimes the parabolic approximation can fail;
        #check if d3 is out of range < d0 or > d2 or the new energy is higher
        if d3 < d0 or d3 > d2 or PE3 > PE0 or PE3 > PE1 or PE3 > PE2:
            #instead, just compute the new distance by bisecting two
            #of the existing points along the line search
            if abs(d2 - d1) > abs(d0 - d1):
                d3 = 0.5 * (d2 + d1)
            else:
                d3 = 0.5 * (d0 + d1)
            PE3 = mdlib.calcenergy(Pos + NormDir * d3, rfix, itype, sigma2ls, epsls, \
                Lx, Ly, Lz, Cut)
            
        #decide which three points to keep; we want to keep
        #the three that are closest to the minimum
        if d3 < d1:
            if PE3 < PE1:
                #get rid of point 2
                Dists, PEs = [d0, d3, d1], [PE0, PE3, PE1]
            else:
                #get rid of point 0
                Dists, PEs = [d3, d1, d2], [PE3, PE1, PE2]
        else:
            if PE3 < PE1:
                #get rid of point 0
                Dists, PEs = [d1, d3, d2], [PE1, PE3, PE2]
            else:
                #get rid of point 2
                Dists, PEs = [d0, d1, d3], [PE0, PE1, PE3]
                
        #check how much we've changed
        if abs(OldPE3 - PE3) < EFracTol * abs(PE3):
            #the fractional change is less than the tolerance,
            #so we are done and can exit the loop
            break
        OldPE3 = PE3

    #return the position array at the minimum (point 1)        
    PosMin = Pos + NormDir * Dists[1]
    PEMin = PEs[1]

    #if using visualization, update the display
    if UseVisual:
        if atomvis.Initialized:
            #update the positions
            atomvis.Update(PosMin)
        else:
            #initialize the visualization window
            atomvis.Init(PosMin)
        
    return PEMin, PosMin

        
def ConjugateGradient(Pos, dx, EFracTolLS, EFracTolCG):
    """Performs a conjugate gradient search.
Input:
    Pos: starting positions, (N,3) array
    dx: initial step amount
    EFracTolLS: fractional energy tolerance for line search
    EFracTolCG: fractional energy tolerance for conjugate gradient
Output:
    PEnergy: value of potential energy at minimum
    Pos: minimum energy (N,3) position array 
"""
    #identify global variables
    global itype, rfix, sigma2ls, epsls, Ntypes, Lx, Ly, Lz, Cut
    #initial search direction
    Forces = np.zeros_like(Pos)
    PE, Forces = mdlib.calcenergyforces(Pos, rfix, itype, sigma2ls, epsls, \
        Lx, Ly, Lz, Cut, Forces)
    Dir = Forces
    OldPE = 1.e300
    #iterative line searches
    while abs(PE - OldPE) > EFracTolCG * abs(PE):
        OldPE = PE
        PE, Pos = LineSearch(Pos, Dir, dx, EFracTolLS)
        OldForces = Forces.copy()
        PE, Forces = mdlib.calcenergyforces(Pos, rfix, itype, sigma2ls, epsls, \
            Lx, Ly, Lz, Cut, Forces)
        Gamma = np.sum((Forces - OldForces) * Forces) / np.sum(OldForces * OldForces)
        Dir = Forces + Gamma *  Dir
    return PE, Pos


def Initialization(rhols, zboundsls, Ntypes):
    """Function that initializes type, position, velocity, and forces arrays
    on the basis of user-supplied subregion bounds, and subregion densities
Input:
    rhols - list of densities for each subregion
    zboundsls - list of different subregion bounds inside which positions/velocities/
        forces will be initialized. Generally, different regions correspond to 
        different LJ fluid type unless identical sigma/eps values specified for
        different regions
    Ntypes - number of different fluid types in system (IMPORTANT: Ntypes is the 
        same as the number of different subregions! Even if we have 2 identical
        fluids, they are counted as different if initialized in different subregions)
Output:
    Pos - position array for all atoms in system
    vel - velocity array for all atoms in system, shifted for zero net momentum
    accel - forces array for all atoms in system
    itype - array describing the type of each atom in multicomponent system
    dx -displacement array for all atoms ins system
    NAtom - total number of atoms in system         
"""
 
    i = 0 #Initialize loop variable
    while i < Ntypes:
        if i == 0:
            #Initializing all arrays:
            rho = rhols[i] #select value for density of current domain
            [zmin,zmax] = zboundsls[i,:] #select current subdomain bounds
            
            Pos, NAtom = InitPositions(rho,Lx,Ly,Lz,zmin,zmax) #initialize positions in subdomain

            vel = InitVelocities(NAtom,T) #initialize velocities in subdomain  
            #Calculating total momentum in subdomain:
            p = sum(vel,0)
            #Shifting velocities in subdomain so that subdomain has zero net momentum:
            vel = vel - p/NAtom
            
            itype = np.zeros(NAtom,dtype=int) #assign type of atoms in subdomain to itype array
            accel = InitAccel(Pos,itype)  #initialize forces in subdomain
            
            i += 1
        else:
            #If multiple fluids, adding additional subdomains to existing arrays:
            rho = rhols[i]
            [zmin,zmax] = zboundsls[i,:]
            
            PosAD, NAtomAD = InitPositions(rho,Lx,Ly,Lz,zmin,zmax) #initialize positions in subdomain
            velAD = InitVelocities(NAtomAD,T) #initialize velocities in subdomain 
            #Calculating total momentum in subdomain:
            p = sum(velAD,0)
            #Shifting velocities so that system has zero net momentum:
            velAD = velAD - p/NAtomAD
            
            itypeAD = np.zeros(NAtomAD,dtype=int) + i
            accelAD = InitAccel(PosAD,itypeAD)  #initialize forces in subdomain

            #Add arrays for subdomain to total system arrays:
            Pos = np.concatenate((Pos,PosAD),axis=0)
            vel = np.concatenate((vel,velAD),axis=0)
            itype = np.concatenate((itype,itypeAD),axis=0)
            accel = np.concatenate((accel,accelAD),axis=0)
            
            NAtom = NAtom + NAtomAD
            i += 1
    
    #Initializing displacement:
    dx = np.zeros([NAtom,3])        
    return Pos, vel, accel, itype, dx, NAtom    
        

def InitPositions(rho, Lx, Ly, Lz, zmin, zmax):
    """Returns array of initial positions for atoms in specified
        subdomain. Subdomains only divided in z direction. 
Input: 
    rho - density of subregion
    Lx - global box dimension, x-direction
    Ly - global box dimension, y-direction
    Lz - global box dimension, z-direction
    zmin - lower bound, z-direction, of subregion
    zmax - upper bound, z-direction, of subregion
Output:
    Pos - position array for atoms generated within subregion
    N - number of atoms generated in subregion
"""
    #Check that user input isn't nonsensical:
    if zmin < 0.0: print "Improper Subdomain Bounds!"
    if zmax > Lz: print "Improper Subdomain Bounds!"
    #Computing subdomain edge lengths:
    zlength = zmax - zmin
    Pos, N = InitPositionsSubregion(rho, Lx, Ly, zlength)
    #Shifting z-coordinates so that subdomain starts at zmin:
    Pos[:,2] = Pos[:,2] + zmin
    return Pos, N    
    
    
def InitPositionsSubregion(rho, rlx, rly, rlz):
    """Returns an array of initial positions of each atom within 
rectangular region with dimensions Lx, Ly, Lz, with atoms placed 
on a cubic lattice.
Input:
    rho: desired number density for region (N/V where V = rlx*rly*rlz)
    rlx: region length in x direction
    rly: region length in y direction
    rlz: region length in z direction
Output:
    Pos: (N,3) array of positions
    N: Number of atoms necessary for desired density in specified region
"""
    
    #Determine necessary number of atoms for desired density/region size:
    N = np.rint(rlx*rly*rlz*rho)
    
    #make the position array
    Pos = np.zeros((N,3), float)
    
    #compute integer grid # of locations for cubic lattice
    LatSpac = (1/rho)**(1./3.)
    NLatx = rlx / LatSpac
    NLaty = rly / LatSpac
    NLatz = rlz / LatSpac

    #make an array of lattice sites
    rx = LatSpac * np.arange(NLatx, dtype=float)
    ry = LatSpac * np.arange(NLaty, dtype=float)
    rz = LatSpac * np.arange(NLatz, dtype=float)
    
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in rx:
        for y in ry:
            for z in rz:
                Pos[i] = np.array([x,y,z], float)
                #add a random offset to help initial minimization
                Offset = 0.1 * LatSpac * (np.random.rand(3) - 0.5)
                Pos[i] = Pos[i] + Offset
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos, N 
    return Pos, N


def RescaleVelocitiesAdHoc(Vel, T):
    """Rescales velocities in the system to the target temperature.
Input:
    Vel: (N,3) array of atomic velocities
    T: target temperature
Output:
    Vel: same as above
"""
    #recenter to zero net momentum (assuming all masses same)
    Vel = Vel - Vel.mean(axis=0)
    #find the total kinetic energy
    KE = 0.5 * np.sum(Vel * Vel)
    #find velocity scale factor from ratios of kinetic energy
    VScale = np.sqrt(1.5 * len(Vel) * T / KE)
    Vel = Vel * VScale
    return Vel  
    
    
def RescaleVelocitiesAndersen(Vel, T, NAtom):
    """Andersen Thermostat - Selects particle at random,
        assigns new velocity to particle drawn randomly from 
        Maxwell-Boltzmann distribution.
    IMPORTANT: Currently Thermostat used at constant time intervals. For more 
        rigorous implementation, particles should have their velocities rescaled
        with a random frequency drawn from Poisson distribution (i.e. every
        particle has probability of having its velocities reassigned proportional to
        nu*tstep at every timestep). 
    Input:
        Vel - Natom-by-3 array of particle velocities
        T - desired system temperature
        NAtom - Number of atoms in system
    Output:
        Vel - Array of particle velocities with updated particle velocity
"""  
    #Select particle at random:
    #i = np.random.random_integers(0,NAtom-1) #this is for all atoms
    i = np.random.random_integers(0, len(itype) - 1) #thermostat wall only
        
    mu = 0
    std = np.sqrt(T)
    Vel[i,:] = np.random.normal(mu, std, 3)
    return Vel
    


def InitVelocities(N, T):
    """Returns an initial random velocity set.
Input:
    N: number of atoms
    T: target temperature
Output:
    Vel: (N,3) array of atomic velocities
"""
    Vel = np.random.rand(N, 3)
    Vel = RescaleVelocitiesAdHoc(Vel, T)
    return Vel


def InitAccel(Pos,subtype):
    """Returns the initial acceleration array.
Input:
    Pos: (N,3) array of atomic positions
Output:
    Accel: (N,3) array of acceleration vectors
"""
    #global variables
    global itype, rfix, sigma2ls, epsls, Ntypes, Lx, Ly, Lz, Cut
    #get the acceleration from the forces
    Forces = np.zeros_like(Pos)
    rfix = Pos.copy() #Assume initial configuration at potential energy min
    PEnergy, Accel = mdlib.calcenergyforces(Pos, rfix, subtype, sigma2ls, epsls, \
        Lx, Ly, Lz, Cut, Forces)
        
    return Accel


def InstTemp(Vel):
    """Returns the instantaneous temperature.
Input:
    Vel: (N,3) array of atomic velocities
Output:
    Tinst: float
"""
    return np.sum(Vel * Vel) / (3. * len(Vel))
    
        
def DensityProfile(Pos,itype,ptype,Nbins,Lz):
    """Function that calculates density profile using binning procedure 
    from supplied particle position array (bins in z-direction)
Input: 
    Pos - NAtom-by-3 position array
    itype - list of particle types associated with Pos
    ptype - particle type of which we desire density profile
    Nbins - number of bins to divide z-length into
    Lz - length of simulation box in z-direction
Output:
    ncount - array of number of particle counts in each bin
""" 
    binsize = Lz/float(Nbins) #size of each bin   
    ncount = np.zeros(Nbins) #make empty array we will add bin counts to
    #Loop through all particles
    i = 0
    while i < len(Pos):
        if itype[i] == ptype:
            binnum = int(np.ceil(Pos[i,2]/binsize)) #bin number selected particle lies in
            ncount[binnum - 1] = ncount[binnum - 1] + 1
        
        i += 1
        
    return ncount

#III. USER INPUT
#-------------------------------------------------#
#### USER INPUT HERE: ####

#========== GLOBAL VARIABLES ==========
#distance cutoff for pairwise interactions
Cut = 5.0
#set the random number seed; useful for debugging
np.random.seed(342324)

#NOTE:
#unit atomic masses assumed (forces = accelerations)
#Global Box Definition:
###########################
Lx = 12.0
Ly = 12.0
Lz = 37.0

#Fluid(s) Parameters
###########################
Ntypes = 3 #number of atom/fluid types
rhols = np.array([1.0,0.6,0.1]) #list of packing for subdomains of differing fluids

#different atom type radii (sigma) HYDROPHILIC
#Build Ntype X Ntype array of sigmai*sigmaj values. Array format:
    #sigma2ls = np.array([[sigma0*sigma0, sigma0*sigma1, sigma0*sigma2],\
    #    [sigma1*sigma0, sigma1*sigma1, sigma1*sigma2],\
    #    [sigma2*sigma0, sigma2*sigma1, sigma2*sigma2]])
sigma2ls = (np.array([[1.0, 1.0, 1.31],\
    [1.0, 1.0, 1.31],\
    [1.31, 1.31, 1.62]]))**2
    
#different atom type energies of interaction HYDROPHILIC 
#Build Ntype X Ntype array of epsij values. Array format:   
    #epsls = np.sqrt(np.array([[eps0*eps0,eps0*eps1,eps0*eps2],\
    #    [eps1*eps0,eps1*eps1,eps1*eps2],\
    #    [eps2*eps0,eps2*eps1,eps2*eps2]]))    
epsls = np.sqrt(np.array([[1.0, 0.7404, 0.4],\
    [0.7404, 1.2, 0.712],\
    [0.4, 0.712, 0.4]]))

#Subdomain Definitions (z-direction, 0 < zmin/zmax < Lz)
###########################
#Array format:
    #zboundsls = np.array([[zmin0,zmax0],[zmin1,zmax1],[zmin2,zmax2]])
zboundsls = np.array([[36.0,37.0],[24.0,36.0],[12.0,24.0]])

#Run Conditions/Parameters
###########################
Cut = 5.0 #Cut-off radius (LJ reduced length)
T = 1.0 #Temperature (LJ reduced units)
tstep = 0.002 #time-step size

NStepsEquil = 100000 #length of equilibration run
NStepsProd = 1000000 #length of production run

RescaleFreq = 50 #frequency for temperature rescaling/collision w/heat bath

#Sampling program parameters:
###########################
Nbins = 90 #number of bins to use in computing density profile
SampleFreq = 50 #frequency for sampling
PdbFreq = 25 #frequency for dumping to trajectory Pdb file

#Print progress?
printprogress = False
progfreq = 100 #frequency for printing progress
#Enable visualization?
UseVisual = False

#Energy minimization parameters:
###########################
EFracTolLS = 1.e-8 #line search tolerance
EFracTolCG = 1.e-10 #conjugate gradient tolerance
dxmin = 0.005 #step size

###END USER INPUT###

#IV. User Input Processing
#-------------------------------------------------#
#Initializing positions/velocities/forces:
Pos, vel, accel, itype, dx, NAtom = Initialization(rhols, zboundsls, Ntypes)
#Store intial positions of pinned atoms as equilibrium positions:
rfix = Pos.copy()

#Energy-minimizing generated particle distribution for given chain length:
print "Performing Energy Minimization..."
PEnergymin, Pos = ConjugateGradient(Pos, dxmin, EFracTolLS, EFracTolCG)
#update forces on basis of new minimized positions:
Penergy,accel = mdlib.calcenergyforces(Pos, rfix, itype, sigma2ls, epsls, \
    Lx, Ly, Lz, Cut, accel)

#Opening files/array for dumping density data:
rhogas = np.zeros(Nbins)
rholiquid = np.zeros(Nbins)
k = file("gasdensity_hydrophilic.txt","w")
l = file("liquiddensity_hydrophilic.txt","w")
g = file("gasdensity_hydrophobic.txt","w") 
h = file("liquiddensity_hydrophobic.txt","w")

#Import visualization module if visualization enabled:
if UseVisual:
    #import custom visualization library
    import atomvis
    
#Prepare list of atom names for PDB file (for post-processing in Chimera):
i = 0
Names = []
while i < len(itype):
    if itype[i] == 0:
        Names.append('C')
    if itype[i] == 1:
        Names.append('H')
    if itype[i] == 2:
        Names.append('O')

    i += 1

#Open Pdb file for writing trajectory:    
Pdb = atomwrite.pdbfile("animLONG.pdb", [Lx,Ly,Lz], AtomNames = Names)

#V. EQUILIBRATION MD LOOP:
#-------------------------------------------------#
step = 0 #initialize tracker
print "Initializing Equilibration MD Loop..."
while step < NStepsEquil:
    #Propagate positions, velocities in time
    Pos, vel, accel, dx, Kenergy, Penergy = mdlib.vvintegrate(Pos, vel, accel, dx, \
        rfix, itype, sigma2ls, epsls, Lx, Ly, Lz, Cut, tstep)
   
    #Thermostat wall:
    if step % RescaleFreq == 0: vel = RescaleVelocitiesAndersen(vel, T, NAtom)
    
    #update time tracker
    step += 1
    
    if step > (NStepsEquil/2):
        #We want to collect data about the density profile, trajectories after system
        #has been equilibrated, but prior to switching on hydrophobicity: 
        #Collect data:
        if step % SampleFreq == 0:
            #Obtain counts in each bin:
            ncount_gas = DensityProfile(Pos,itype,2,Nbins,Lz)
            ncount_liquid = DensityProfile(Pos,itype,1,Nbins,Lz)
            #Loop through arrays for proper output formatting:
            ind = 0
            while ind < Nbins:
                k.write(str(ncount_gas[ind]) + " ")
                l.write(str(ncount_liquid[ind]) + " ")
                ind += 1
            
            #Start next line for next writing operation
            k.write('\n')
            l.write('\n')
        
    #display progress if enabled:
    if printprogress:
        if step % progfreq == 0:
            runprogress = float(step)/float(NStepsEquil)*100.
            Energytot = Kenergy + Penergy
            Temp = InstTemp(vel)

            print "%05.2f" %runprogress + " " + \
            "%08.3f" %Energytot  + " " + "%08.3f" %Penergy + " " + "%08.3f" %Kenergy + \
            " " + "%08.3f" %Temp
    
    #update display if visualization enabled
    if UseVisual:
        if atomvis.Initialized:
            #update the positions
            atomvis.Update(Pos)
        else:
            #initialize the visualization window
            atomvis.Init(Pos)
            

#Initializing displacement vector at current positions:
dx = np.zeros([NAtom,3])

#Switch on wall hydrophobicity:
#################    
#different atom type radii (sigma) HYDROPHOBIC
#Build Ntype X Ntype array of sigmai*sigmaj values. Array format:
    #sigma2ls = np.array([[sigma0*sigma0, sigma0*sigma1, sigma0*sigma2],\
    #    [sigma1*sigma0, sigma1*sigma1, sigma1*sigma2],\
    #    [sigma2*sigma0, sigma2*sigma1, sigma2*sigma2]])
sigma2ls = (np.array([[1.0, 1.0, 1.31],\
    [1.0, 1.0, 1.31],\
    [1.31, 1.31, 1.62]]))**2
    
#different atom type energies of interaction HYDROPHOBIC   
#Build Ntype X Ntype array of epsij values. Array format:
    #epsls = np.sqrt(np.array([[eps0*eps0,eps0*eps1,eps0*eps2],\
    #    [eps1*eps0,eps1*eps1,eps1*eps2],\
    #    [eps2*eps0,eps2*eps1,eps2*eps2]]))    
epsls = np.sqrt(np.array([[1.0, 0.3996, 0.4],\
    [0.3996, 1.2, 0.712],\
    [0.4, 0.712, 0.4]]))

#VI. PRODUCTION MD LOOP:
#-------------------------------------------------#
print 'Initializing Production MD loop...'
step = 0 #Initializing tracker
while step < NStepsProd:
    #Propagate positions, velocities in time
    Pos, vel, accel, dx, Kenergy, Penergy = mdlib.vvintegrate(Pos, vel, accel, dx, \
        rfix, itype, sigma2ls, epsls, Lx, Ly, Lz, Cut, tstep)
   
    #Thermostat wall:
    if step % RescaleFreq == 0: vel = RescaleVelocitiesAndersen(vel, T, NAtom)
    
    #update time tracker
    step += 1
    
    #Dump trajectory data:
    if step % PdbFreq == 0: 
        Pdb.write(Pos) 
    
    #Collect data:
    if step % SampleFreq == 0:        
        #Obtain counts in each bin:
        ncount_gas = DensityProfile(Pos,itype,2,Nbins,Lz)
        ncount_liquid = DensityProfile(Pos,itype,1,Nbins,Lz)
        #Loop through arrays for proper output formatting:
        ind = 0
        while ind < Nbins:
            g.write(str(ncount_gas[ind]) + " ")
            h.write(str(ncount_liquid[ind]) + " ")
            ind += 1
            
        g.write('\n')
        h.write('\n')
    
    
    #display progress if enabled:
    if printprogress:
        if step % progfreq ==0:
            runprogress = float(step)/float(NStepsProd)*100.
            Energytot = Kenergy + Penergy
            Temp = InstTemp(vel)

            print "%05.2f" %runprogress + " " + \
            "%08.3f" %Energytot  + " " + "%08.3f" %Penergy + " " + "%08.3f" %Kenergy + \
            " " + "%08.3f" %Temp 
    
    #update display if visualization enabled
    if UseVisual:
        if atomvis.Initialized:
            #update the positions
            atomvis.Update(Pos)
        else:
            #initialize the visualization window
            atomvis.Init(Pos)

    

#Closing all files/visualization:
if UseVisual: atomvis.Close()
k.close()
l.close()
g.close()
h.close()
Pdb.close()